#! /bin/ksh

###################################################################################################
#
# GLOFAS seasonal forecast processing script 1: 
# 
# Run the months in parrallel to:
# Extract the gridded daily seasonal river flow forecast data (raw output from Lisflood)
# Average daily seasonal gridded discharge forecasts over weeks for all ensemble members
#
# (file paths have been changed)
###################################################################################################

year=$1

month=$2 

month=`printf "%02d\n" $month` 

####################################################################################################

cd /tmpdir/

mkdir ./${year}${month}

echo "Extracting data"

#for each ensemble member
for ensmem in $(seq 0 51) ; do

    ensmem=`printf "%02d\n" $ensmem` 
 
    mkdir /tmpdir/${year}${month}/e${ensmem}
    cd /tmpdir/${year}${month}/e${ensmem}

    #copy lisflood output for this ensemble member into tmpdir
    cp /lisflood_output_path/${year}${month}01/EGE${year}${month}0100/p${ensmem}/results/dis00000.* /tmpdir/${year}${month}/e${ensmem}/

    ##################################################################################################################################
  
    #This section averages the daily gridded lisflood river flow output over the weeks (from Monday to Sunday) to get 17 gridded weekly-averaged forecast files:

    module load python

    echo "Computing weekly averages for ensemble member: " ${ensmem}

    cd /weekly_averages/save_dir/
    mkdir ./${year}${month}
    cd ./${year}${month}/
    mkdir ./e${ensmem}

    cd /tmpdir/${year}${month}/e${ensmem}/

    aa=(001 008 015 022 029 036 043 050 057 064 071 078 085 092 099 106 113)
    bb=(002 009 016 023 030 037 044 051 058 065 072 079 086 093 100 107 114)
    cc=(003 010 017 024 031 038 045 052 059 066 073 080 087 094 101 108 115)
    dd=(004 011 018 025 032 039 046 053 060 067 074 081 088 095 102 109 116)
    ee=(005 012 019 026 033 040 047 054 061 068 075 082 089 096 103 110 117)
    ff=(006 013 020 027 034 041 048 055 062 069 076 083 090 097 104 111 118)
    gg=(007 014 021 028 035 042 049 056 063 070 077 084 091 098 105 112 119)
    
    for ((i = 0; i < 17; i++))  ; do
        
        week=`printf "%02d\n" $i` 
        
        #calculate weekly averages of daily pcraster files (file extension .xxx is the forecast day)
        #gdal_calc.py is a gdal sript which does calculations over multiple pcraster files
        
        /script/dir/gdal_calc.py -A dis00000.${aa[i]} -B dis00000.${bb[i]} -C dis00000.${cc[i]} -D dis00000.${dd[i]} -E dis00000.${ee[i]} -F dis00000.${ff[i]} -G dis00000.${gg[i]} --outfile=/weekly_averages/save_dir/${year}${month}/e${ensmem}/wa_gridded_dis_fcst_e${ensmem}_wk${week}.map --calc="(A+B+C+D+E+F+G)/7"

    done

    echo "Finished computing weekly averages for ensemble member: " ${ensmem}
    
done

#clean up the tmpdir to make room for the next forecast

rm -rf /tmpdir/${year}${month}






