import numpy as np
from osgeo import gdal, gdal_array
from datetime import tzinfo, timedelta, datetime,date, time
import datetime as datetime
import gzip
import os
import shutil
import sys
from datetime import datetime

####################################################################################################################################################
# GloFAS seasonal forecast processing script #4:
#
# For one forecast, average the weekly-averaged gridded exceedance probabilities (out to 17 weeks lead time) over the 306 basins
#
# Saves files with all basins and all weeks' exceedance probs (306 rows, 18 columns - 1 for basin ID, 17 for weeks)
#
#(file paths have been changed)
####################################################################################################################################################


def compute_basin_mean(year, month):

    bn = 306 #number of basins
    wn = 17 #number of weeks
    pctls=[20,80]
    minups = 1500 #the minimum upstream area shown on the interface is 1500km2


    #datadir = '/scratch/mo/more/GLOFAS_SEASONAL/SEAS5_REFORECASTS/glofas2.1seas5_fcprod/wexcprobs/'+str(year)+str(month).zfill(2)+'/'
    #MASKED exceednace probs:
    datadir = "/weekly_averages/save_dir/"+str(year)+str(month).zfill(2)+"/"

    savedir = '/basin_averages/save_dir/'+str(year)+str(month).zfill(2)+'/'
    if not os.path.exists(savedir):
            os.makedirs(savedir)
            
    #pcraster file containing the upstream area at every grid point
    upsfile = '/archive/ups.map'
    ff = gdal.Open(upsfile)
    rows = ff.RasterYSize
    cols = ff.RasterXSize
    band = ff.GetRasterBand(1)
    ups = band.ReadAsArray(0,0,cols,rows)
    ups = np.flipud(ups)
    
    #array to store the basin number and averages for all weeks (306 rows, 18 columns, first is the basin number)
    bav_allweeks = np.zeros((bn,wn+1)) 
    
    for p in pctls:
                   
        for w in range(wn): #loop over each week of lead time from 0 to 16
    
            #read the exceedance probabilities into an array
            inputfile = datadir+"prob_"+str(year)+str(month).zfill(2)+"01_w"+str(w+1).zfill(2)+"_p"+str(p)+"m.map"
            ff = gdal.Open(inputfile)
            rows = ff.RasterYSize
            cols = ff.RasterXSize
            band = ff.GetRasterBand(1)
            data = band.ReadAsArray(0,0,cols,rows)
            data = np.flipud(data)
            data[data < 0] = np.nan
                    
            #read the rasterised major river basins shapefile
            shapefile = '/archive/glofas_seasonal_shapefile_rasterized.tiff'
            shpff = gdal.Open(shapefile)
            shprows = shpff.RasterYSize
            shpcols = shpff.RasterXSize
            shpband = shpff.GetRasterBand(1)
            shpraster = shpband.ReadAsArray(0,0,shpcols,shprows)
            #print shpraster

            numrows = 1500
            numcols = 3600
                
            #####################
            #calculates weekly basin averages
            #####################
            
            basinQ_temp = dict() #empty dictionary to temporarily store the gridded probabilities for this week, organised by basin

            for fid in range(1,bn+1):
                basinQ_temp[fid] = [] #empty lists within dictionary to store the gridded probabilities within each basin
                
            #loops over the gridpoints in the basin shapefile to allocate gridded probabilities to corresponding basin's list
            for r in range(numrows):
                for c in range(numcols):
                    #if the shapefile for this grid point has a basin number, and the upstream area is above our minimum, append the probability to the correct basin list:
                    if (shpraster[r,c] in np.arange(1,bn+1)) and (ups[r,c] >= minups):
                        basinQ_temp[shpraster[r,c]].append(data[r,c])
                        
            #remove any nan values in the lists (using a temporary list) and take the average of the remaining values 
            for fid in range(1,bn+1):
                bav_allweeks[fid-1,0] = fid
                if len(basinQ_temp[fid]) == 0:
                    bav_allweeks[fid-1,w+1] = np.nan
                else:
                    cleanedtemp = [x for x in basinQ_temp[fid] if str(x) != 'nan'] 
                    if len(cleanedtemp) == 0: 
                        bav_allweeks[fid-1,w+1] = np.nan 
                    else:
                        bav_allweeks[fid-1,w+1] = sum(cleanedtemp)/float(len(cleanedtemp))


        savefile = savedir+'ba_weekly_exc_probs'+str(year)+str(month).zfill(2)+'_pctl'+str(p)+'.txt'
        np.savetxt(savefile, bav_allweeks, fmt="%s")


if __name__ == '__main__': 
    compute_basin_mean(int(sys.argv[1]), int(sys.argv[2]))
