import sys
from osgeo import gdal, gdal_array
import gzip
import numpy as np
import os
import matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
from matplotlib.patches import Polygon
from matplotlib.collections import PatchCollection
from matplotlib.patches import PathPatch
import matplotlib.patches as mpatches
from datetime import datetime

####################################################################################################################################################
# GLOFAS seasonal forecast processing script #6:
#
# For one forecast, this script creates an image of the basin overview layer map
#
#(file paths have been changed)
####################################################################################################################################################


def create_forecast_map(year, month):
    
    bn=306
    
    #load in the max probabilities to plot on the map
    probdir = '/basin_averages/save_dir/'+str(year)+str(month).zfill(2)+'/' 
    savedir = '/map_vis/save_dir/'+str(year)+str(month).zfill(2)+'/' 
    if not os.path.exists(savedir):
        os.makedirs(savedir)
    
    max_prob_file = probdir+'ba_max_probs_4mon_overview_BOTH_'+str(year)+str(month).zfill(2)+'.txt'
    max_prob = np.genfromtxt(max_prob_file)

    
    maps = ["both"] #can be used to just plot an orange or blue map rather than both colours
    
    for i in maps:
               
        if i == "both":
            max_prob = max_prob_both*100
        elif i == "blue":
            max_prob = max_prob_blue*100
        elif i == "orange":
            max_prob = max_prob_orange*100
            
        white=[]
        darkblue=[]
        midblue=[]
        lightblue=[]
        lightorange=[]
        midorange=[]
        darkorange=[]
    
        for b in range(bn):
            
            #assign a colour according to the probability of exceeding the upper and lower thresholds
    
            if np.isnan(max_prob_both[b]):
                white.append(b+1)
            
            elif max_prob[b] >= 90:
                darkblue.append(b+1)
            
            elif max_prob[b] >= 75 and max_prob[b] < 90:
                midblue.append(b+1)
            
            elif max_prob[b] >= 50 and max_prob[b] < 75:
                lightblue.append(b+1)
            
            elif max_prob[b] > -50 and max_prob[b] < 50:
                white.append(b+1)
            
            elif max_prob[b] <= -50 and max_prob[b] > -75:
                lightorange.append(b+1)
            
            elif max_prob[b] <= -75 and max_prob[b] > -90:
                midorange.append(b+1)
            
            elif max_prob[b] <=-90:
                darkorange.append(b+1)
                
            else:
                print "Error - max prob didn't fit in any category :( Basin: ",b
                
        fig = plt.figure()
        fig.set_size_inches(18, 10)
        ax = fig.add_subplot(111)
        lat1 = 75
        lat2 = -75
        lon1 = -180
        lon2 = 180
        m = Basemap(llcrnrlon=lon1,llcrnrlat=lat2,urcrnrlon=lon2,urcrnrlat=lat1,projection='mill',resolution='l')

        #read in the shapefile with the glofas-seasonal basins:
        m.readshapefile('glofas_seasonal_shapefile_noatts', 'glofas_seasonal_shapefile')

        whitebasins=[]
        darkbluebasins=[]
        midbluebasins=[]
        lightbluebasins=[]
        darkorangebasins=[]
        midorangebasins=[]
        lightorangebasins=[]


        for bid,shape in zip(m.glofas_seasonal_shapefile_info, m.glofas_seasonal_shapefile):

            if bid['ID'] in darkblue:
                darkbluebasins.append( Polygon(np.array(shape), True) )
            elif bid['ID'] in midblue:
                midbluebasins.append( Polygon(np.array(shape), True) )
            elif bid['ID'] in lightblue:
                lightbluebasins.append( Polygon(np.array(shape), True) )
            elif bid['ID'] in darkorange:
                darkorangebasins.append( Polygon(np.array(shape), True) )
            elif bid['ID'] in midorange:
                midorangebasins.append( Polygon(np.array(shape), True) )
            elif bid['ID'] in lightorange:
                lightorangebasins.append( Polygon(np.array(shape), True) )
            elif bid['ID'] in white:
                whitebasins.append( Polygon(np.array(shape), True) )

            else:
                whitebasins.append( Polygon(np.array(shape), True) )
                
        ax.add_collection(PatchCollection(whitebasins, facecolor= 'white', edgecolor='k', alpha=1, linewidths=0.25))
        ax.add_collection(PatchCollection(darkbluebasins, facecolor= 'mediumblue', edgecolor='k', alpha=0.9, linewidths=0.25))
        ax.add_collection(PatchCollection(midbluebasins, facecolor= 'mediumblue', edgecolor='k', alpha=0.6, linewidths=0.25))
        ax.add_collection(PatchCollection(lightbluebasins, facecolor= 'mediumblue', edgecolor='k',alpha=0.2, linewidths=0.25))
        ax.add_collection(PatchCollection(darkorangebasins, facecolor= '#e87c08', edgecolor='k', alpha=0.9, linewidths=0.25)) #crimson #f39c12
        ax.add_collection(PatchCollection(midorangebasins, facecolor= '#e87c08', edgecolor='k', alpha=0.6, linewidths=0.25))
        ax.add_collection(PatchCollection(lightorangebasins, facecolor= '#e87c08', edgecolor='k', alpha=0.25, linewidths=0.25))

        #bluehigh = mpatches.Patch(color='mediumblue', alpha=0.9, label='>  90% probability of high flows')
        #bluemed = mpatches.Patch(color='mediumblue',alpha=0.6, label='75-90% probability of high flows')
        #bluelow = mpatches.Patch(color='mediumblue', alpha=0.2, label='50-75% probability of high flows')
        #orangelow = mpatches.Patch(color='#e87c08', alpha=0.25, label='50-75% probability of low flows')
        #orangemed = mpatches.Patch(color='#e87c08',alpha=0.6, label='75-90% probability of low flows')
        #orangehigh = mpatches.Patch(color='#e87c08', alpha=0.9, label='>  90% probability of low flows')
        
        #if i == "both":
            #white = mpatches.Patch(facecolor='white', edgecolor='k',alpha=1,lw=0.5, label=' ')
            #leg1=plt.legend(handles=[bluehigh,bluemed,bluelow,white,orangelow,orangemed,orangehigh], fontsize = 12, loc='lower left', frameon=False)
        #elif i == "blue":
            #white = mpatches.Patch(facecolor='white', edgecolor='k',alpha=1,lw=0.5, label='< 50% probability of high flows')
            #leg1=plt.legend(handles=[bluehigh,bluemed,bluelow,white], fontsize = 12, loc='lower left', frameon=False)
        #elif i == "orange":
            #white = mpatches.Patch(facecolor='white', edgecolor='k',alpha=1,lw=0.5, label='< 50% probability of low flows')
            #leg1=plt.legend(handles=[orangehigh,orangemed,orangelow,white], fontsize = 12, loc='lower left', frameon=False)
            
        plt.savefig(savedir+'seas_forecast_map_4mon_overview'+str(year)+str(month).zfill(2)+'_'+str(i)+'.png', bbox_inches=0, dpi=800)

if __name__ == '__main__': 
    create_forecast_map(int(sys.argv[1]), int(sys.argv[2]))
            
            
    
            
            
            
            
            
            
            
            
            
            
