# Convert PCRaster maps into graphic format (gif)

####################################################################################################################################################
# GLOFAS seasonal forecast processing script #7:
#
# For one forecast, this script creates the coloured river network layer map for the interface
#
#(file paths have been changed)
####################################################################################################################################################


args <- commandArgs(TRUE)

yy <- as.numeric(args[1])       # year
mm <- as.numeric(args[2])       # month

library(rgdal)
library(caTools)

#supporting data directory - where is the ups.map?
dataDir <- as.character('/script_archive/') 

#forecast data directory - data to display (i.e. max exceedance probabilities)
rootDir <- as.character("/basin_averages/save_dir/")

saveDir <- as.character('/map_vis/save_dir/')

col_glofas_seas<-c(
rep(rgb(220/255,220/255,220/255),1), #grey
rep(rgb(0/255,0/255,128/255),1), #dark blue
rep(rgb(65/255,105/255,255/255),1), #medium blue
rep(rgb(135/225,206/255,250/255),1), #light blue
rep(rgb(255/255,204/255,153/255),1), #light orange
rep(rgb(255/255,165/255,0/255),1), #medium orange
rep(rgb(210/255,105/255,30/255),1), #dark orange
rep(rgb(255/255,255/255,255/255),1), #white
rep(rgb(139/255,131/255,120/255),1), #dark grey (for arid areas with odd forecasts)
rep(grey(0), 256-9))#???

#leading zeroes
yy0=sprintf("%04.0f",yy)
mm0=sprintf("%02.0f",mm)

minups <- 1500

#load in the upstream area file (raster map)
ups <- as.matrix(readGDAL(paste(dataDir,"/ups.map",sep="")))


#---------------------------------
#FOR BOTH ORANGE AND BLUE
#---------------------------------

#read in the forecast file (max exceedance probabilities, raster map)
maxexcprob <- as.matrix(readGDAL(paste0(rootDir,"/",yy0,mm0,"/masked_max_probs_interface_",yy0,mm0,"_fcst_4month_overview.tiff")))
maxexcprob <- maxexcprob*100

#loop over the grid points
#assign colours  (RGB) depending on criteria
#0=grey, 1=dark blue, 2=medium blue, 3=light blue, 4=light orange, 5=medium orange, 6=dark orange, 7=white, 8=dark grey (to use for arid areas with weird forecasts)

for(i in 1:3600) {
	for(j in 1:1500) {
        #if the probability and the upstream area are nan (i.e. the ocean), set to 255 (which is set to transparent when saving)
		if (is.na(maxexcprob[i,j])) {maxexcprob[i,j]<-8} #if the upstream area has a value (not ocean), but the max prob is nan, then it's an arid area to mask
		#otherwise, for the pixels that aren't masked, assign a colour according to the probability of exceeding the upper and lower thresholds
		else if (maxexcprob[i,j] >= 90) {maxexcprob[i,j]<-1}
        else if (maxexcprob[i,j] >= 75 & maxexcprob[i,j] < 90) {maxexcprob[i,j]<-2}
        else if (maxexcprob[i,j] >= 50 & maxexcprob[i,j] < 75) {maxexcprob[i,j]<-3}
        else if (maxexcprob[i,j] > -50 & maxexcprob[i,j] < 50) {maxexcprob[i,j]<-0}
        else if (maxexcprob[i,j] <= -50 & maxexcprob[i,j] > -75) {maxexcprob[i,j]<-4}
        else if (maxexcprob[i,j] <= -75 & maxexcprob[i,j] > -90) {maxexcprob[i,j]<-5}
        else if (maxexcprob[i,j] <= -90) {maxexcprob[i,j]<-6}
        else {cat("ERROR",i,j,maxexcprob[i,j])}

        if (is.na(ups[i,j])) {maxexcprob[i,j]<-7} #if the upstream area is nan, it's the ocean - set to white (transparent)
        else if (ups[i,j] < minups) {maxexcprob[i,j]<-7}  #also, if the upstream area is below our minimum - set to white (transparent)
    
    }
}


maxexcprob[is.na(maxexcprob)]<-255 #if any of it is nan, set to transparent
#save the image:
write.gif(image=t(maxexcprob), filename=paste(saveDir,"/",yy0,mm0,"/max_exc_prob_coloured_river_network_map_transparent_",yy0,mm0,".gif",sep=""),col=col_glofas_seas,scale="never",transparent=0)

write.gif(image=t(maxexcprob), filename=paste(saveDir,"/",yy0,mm0,"/max_exc_prob_coloured_river_network_map_white_",yy0,mm0,".gif",sep=""),col=col_glofas_seas,scale="never")



